@@ -272,6 +272,8 @@ _cogl_atlas_create_texture (CoglAtlas *atlas,
 {
   CoglHandle tex;
 
+  _COGL_GET_CONTEXT (ctx, COGL_INVALID_HANDLE);
+
   if ((atlas->flags & COGL_ATLAS_CLEAR_TEXTURE))
     {
       guint8 *clear_data;
@@ -280,22 +282,22 @@ _cogl_atlas_create_texture (CoglAtlas *atlas,
 
       /* Create a buffer of zeroes to initially clear the texture */
       clear_data = g_malloc0 (width * height * bpp);
-      clear_bmp = _cogl_bitmap_new_from_data (clear_data,
-                                              atlas->texture_format,
-                                              width,
-                                              height,
-                                              width * bpp,
-                                              (CoglBitmapDestroyNotify) g_free,
-                                              NULL);
+      clear_bmp = cogl_bitmap_new_for_data (ctx,
+                                            width,
+                                            height,
+                                            atlas->texture_format,
+                                            width * bpp,
+                                            clear_data);
 
       tex = _cogl_texture_2d_new_from_bitmap (clear_bmp, COGL_TEXTURE_NONE,
                                               atlas->texture_format,
                                               NULL);
       cogl_object_unref (clear_bmp);
+
+      g_free (clear_data);
     }
   else
     {
-      _COGL_GET_CONTEXT (ctx, COGL_INVALID_HANDLE);
       tex = cogl_texture_2d_new_with_size (ctx,
                                            width, height,
                                            atlas->texture_format,
