@@ -37,12 +37,6 @@ static void _cogl_swap_chain_free (CoglSwapChain *swap_chain);
 
 COGL_OBJECT_DEFINE (SwapChain, swap_chain);
 
-GQuark
-cogl_swap_chain_error_quark (void)
-{
-  return g_quark_from_static_string ("cogl-swap-chain-error-quark");
-}
-
 static void
 _cogl_swap_chain_free (CoglSwapChain *swap_chain)
 {
