@@ -13,10 +13,8 @@
 
 typedef struct _TestState
 {
-  CoglContext *ctx;
   int fb_width;
   int fb_height;
-  CoglFramebuffer *fb;
 } TestState;
 
 static CoglTexture3D *
@@ -75,8 +73,8 @@ create_texture_3d (CoglContext *context)
 static void
 draw_frame (TestState *state)
 {
-  CoglTexture *tex = COGL_TEXTURE (create_texture_3d (state->ctx));
-  CoglPipeline *pipeline = cogl_pipeline_new (state->ctx);
+  CoglTexture *tex = COGL_TEXTURE (create_texture_3d (ctx));
+  CoglPipeline *pipeline = cogl_pipeline_new (ctx);
   typedef struct { float x, y, s, t, r; } Vert;
   CoglPrimitive *primitive;
   CoglAttributeBuffer *attribute_buffer;
@@ -89,15 +87,13 @@ draw_frame (TestState *state)
   cogl_pipeline_set_layer_filters (pipeline, 0,
                                    COGL_PIPELINE_FILTER_NEAREST,
                                    COGL_PIPELINE_FILTER_NEAREST);
-  cogl_push_source (pipeline);
 
   /* Render the texture repeated horizontally twice using a regular
      cogl rectangle. This should end up with the r texture coordinates
      as zero */
-  cogl_rectangle_with_texture_coords (0.0f, 0.0f, TEX_WIDTH * 2, TEX_HEIGHT,
-                                      0.0f, 0.0f, 2.0f, 1.0f);
-
-  cogl_pop_source ();
+  cogl_framebuffer_draw_textured_rectangle (fb, pipeline,
+                                            0.0f, 0.0f, TEX_WIDTH * 2, TEX_HEIGHT,
+                                            0.0f, 0.0f, 2.0f, 1.0f);
 
   /* Render all of the images in the texture using coordinates from a
      CoglPrimitive */
@@ -135,7 +131,7 @@ draw_frame (TestState *state)
       v++;
     }
 
-  attribute_buffer = cogl_attribute_buffer_new (state->ctx,
+  attribute_buffer = cogl_attribute_buffer_new (ctx,
                                                 4 * TEX_DEPTH * sizeof (Vert),
                                                 verts);
   attributes[0] = cogl_attribute_new (attribute_buffer,
@@ -156,11 +152,11 @@ draw_frame (TestState *state)
                                                   2 /* n_attributes */);
 
   cogl_primitive_set_indices (primitive,
-                              cogl_get_rectangle_indices (state->ctx,
+                              cogl_get_rectangle_indices (ctx,
                                                           TEX_DEPTH),
                               6 * TEX_DEPTH);
 
-  cogl_framebuffer_draw_primitive (state->fb, pipeline, primitive);
+  cogl_framebuffer_draw_primitive (fb, pipeline, primitive);
 
   g_free (verts);
 
@@ -178,7 +174,8 @@ validate_block (int block_x, int block_y, int z)
 
   for (y = 0; y < TEX_HEIGHT; y++)
     for (x = 0; x < TEX_WIDTH; x++)
-      test_utils_check_pixel_rgb (block_x * TEX_WIDTH + x,
+      test_utils_check_pixel_rgb (fb,
+                                  block_x * TEX_WIDTH + x,
                                   block_y * TEX_HEIGHT + y,
                                   255 - x * 8,
                                   y * 8,
@@ -204,20 +201,20 @@ test_multi_texture (TestState *state)
   CoglTexture2D *tex_2d;
   guint8 tex_data[4];
 
-  cogl_framebuffer_clear4f (state->fb, COGL_BUFFER_BIT_COLOR, 0, 0, 0, 1);
+  cogl_framebuffer_clear4f (fb, COGL_BUFFER_BIT_COLOR, 0, 0, 0, 1);
 
   /* Tests a pipeline that is using multi-texturing to combine a 3D
      texture with a 2D texture. The texture from another layer is
      sampled with TEXTURE_? just to pick up a specific bug that was
      happening with the ARBfp fragend */
 
-  pipeline = cogl_pipeline_new (state->ctx);
+  pipeline = cogl_pipeline_new (ctx);
 
   tex_data[0] = 0xff;
   tex_data[1] = 0x00;
   tex_data[2] = 0x00;
   tex_data[3] = 0xff;
-  tex_2d = cogl_texture_2d_new_from_data (state->ctx,
+  tex_2d = cogl_texture_2d_new_from_data (ctx,
                                           1, 1, /* width/height */
                                           COGL_PIXEL_FORMAT_RGBA_8888_PRE,
                                           COGL_PIXEL_FORMAT_RGBA_8888_PRE,
@@ -230,7 +227,7 @@ test_multi_texture (TestState *state)
   tex_data[1] = 0xff;
   tex_data[2] = 0x00;
   tex_data[3] = 0xff;
-  tex_3d = cogl_texture_3d_new_from_data (state->ctx,
+  tex_3d = cogl_texture_3d_new_from_data (ctx,
                                           1, 1, 1, /* width/height/depth */
                                           COGL_PIXEL_FORMAT_RGBA_8888_PRE,
                                           COGL_PIXEL_FORMAT_RGBA_8888_PRE,
@@ -247,11 +244,9 @@ test_multi_texture (TestState *state)
                                    "RGBA = ADD(TEXTURE_0, TEXTURE_1)",
                                    NULL);
 
-  cogl_push_source (pipeline);
-  cogl_rectangle (0, 0, 10, 10);
-  cogl_pop_source ();
+  cogl_framebuffer_draw_rectangle (fb, pipeline, 0, 0, 10, 10);
 
-  test_utils_check_pixel (5, 5, 0xffff00ff);
+  test_utils_check_pixel (fb, 5, 5, 0xffff00ff);
 
   cogl_object_unref (tex_2d);
   cogl_object_unref (tex_3d);
@@ -259,23 +254,18 @@ test_multi_texture (TestState *state)
 }
 
 void
-test_cogl_texture_3d (TestUtilsGTestFixture *fixture,
-                      void *data)
+test_texture_3d (void)
 {
-  TestUtilsSharedState *shared_state = data;
-
   /* Check whether GL supports the rectangle extension. If not we'll
      just assume the test passes */
-  if (cogl_has_feature (shared_state->ctx, COGL_FEATURE_ID_TEXTURE_3D))
+  if (cogl_has_feature (ctx, COGL_FEATURE_ID_TEXTURE_3D))
     {
       TestState state;
 
-      state.ctx = shared_state->ctx;
-      state.fb_width = cogl_framebuffer_get_width (shared_state->fb);
-      state.fb_height = cogl_framebuffer_get_height (shared_state->fb);
-      state.fb = shared_state->fb;
+      state.fb_width = cogl_framebuffer_get_width (fb);
+      state.fb_height = cogl_framebuffer_get_height (fb);
 
-      cogl_framebuffer_orthographic (shared_state->fb,
+      cogl_framebuffer_orthographic (fb,
                                      0, 0, /* x_1, y_1 */
                                      state.fb_width, /* x_2 */
                                      state.fb_height /* y_2 */,
